data {
    int N;
    int M;
    array[N, M] int<lower = 1, upper = 3> X;
}
transformed data {
    real C1;
    real C2;
    //
    //     These values specify the origin and unit of the scale
    //     cf. Okamoto (2013). Behaviormetrika, 40, 149-168.
    //
    C1 = -1;
    C2 = 1;
}
parameters {
    array[M] real mu;
    array[M] real<lower = 0.0> Lmd;
    array[M] real<lower = 0.0> vpsy;
    array[N] real F;
}
transformed parameters {
    array[M] real psy;
    array[N, M] vector[3] p;
    array[N, M] real cumP;
    for (j in 1:M) {
        psy[j] = 0.01 + vpsy[j];
    }
    for (i in 1:N)
        for (j in 1:M){
            cumP[i][j] = normal_cdf((C2 - (mu[j] + Lmd[j] * F[i])) / psy[j] | 0.0, 1.0);
            p[i][j][1] = normal_cdf((C1 - (mu[j] + Lmd[j] * F[i])) / psy[j] | 0.0, 1.0);
            p[i][j][2] = cumP[i][j] - p[i][j][1];
            p[i][j][3] = 1.0 - cumP[i][j];
        }
    
}
model {
    for (j in 1:M) {
        mu[j] ~ normal(0.0, 10.0);
        vpsy[j] ~ exponential(0.1);
        Lmd[j] ~ exponential(0.1);
    }
    F ~ normal(0.0, 1.0);
    for (i in 1:N) {
        for (j in 1:M) {
            X[i][j] ~ categorical(p[i][j]);
        }
    }
}
generated quantities {
    real sdF;
    real mnF;
    real sumF;
    real ssF;
    array[N] real cF;
    array[M] real cLmd;
    sumF = 0.0;
    ssF = 0.0;
    for (i in 1:N) {
        sumF += F[i];
        ssF += pow(F[i], 2);
    }
    mnF = sumF / N;
    sdF = pow((ssF - N * pow(mnF, 2)) / (N - 1), 0.5);
    for (i in 1:N) {
        cF[i] = (F[i] - mnF) / sdF;
    }
    for (j in 1:M) {
        cLmd[j] = Lmd[j] * sdF;
    }
}
